'                       SPIN Operators V1.00

'This program 



CON
  _clkmode    = xtal1 + pll16x
  _xinfreq    = 5_000_000

  vgaPinGroup = 8                             ' Base pin for VGA pin group
  ps2KeyData  = 26                             ' Data pin for PS/2 keyboard
  ps2KeyClock = 27                             ' Clock pin for PS/2 keyboard


  
OBJ
  ldr     : "OS_loader"                        ' SPIN program loader and support routines
  ldrInit : "OS_loaderInit"                    '  Initialization and assembly portion
  text    : "OS_vgaHiresText"                  ' Hires VGA driver modified for OS use
  vgaInit : "OS_vgaHiresTextInit"              '  Initialization and assembly portion
  key     : "OS_keyboardA"                      ' PS/2 keyboard driver modified for OS use
  keyInit : "OS_keyboardInitA"                  '  Initialization and assembly portion

  
CON
  _free = (ldr#endMemory - ldr#endFree + ldr#totalNeeded + 3) / 4  ' # longs needed

VAR
  long selection, z
  word a
  byte  newkey,x, buffer[32]

  
PUB initialize | t, i, p
  if long[ldr#initMarker] <> ldr#uniqueMark                ' Check for a valid initialization
    repeat i from 0 to 7
      if i <> cogid                                        ' Stop all cogs except this one
        cogstop(i)
    long[ldr#memPtr]    := ldr#endFree                     ' Start at end of HUB memory
    p := ldr.allocatePRI(ldr#systemSize)                   ' Allocate system work area
    long[ldr#systemPtr] := p
    long[p+ldr#sys_cog] := |<cogid                         ' Mark current cog in use
    byte[p+ldr#sys_cogMain] := cogid + 1
    long[p+ldr#sys_random] := ldr#initMarker               ' Initialize random seed
    wordfill(p+ldr#sys_eeproms,0,ldr#maxPinPairs)          ' Clear EEPROMs bitmap table
    long[ldr#videoPtr]  := ldr.allocatePRI(ldr#videoSize)  ' Allocate work area for video
    long[ldr#screenPtr] := ldr.allocatePRI(ldr#screenSize) ' Allocate screen buffer
    long[ldr#colorPtr]  := ldr.allocatePRI(ldr#colorSize)  ' Allocate screen color buffer
    long[ldr#fontPtr]   := ldr.allocatePRI(ldr#fontSize)   ' Allocate font table
    long[ldr#cursorPtr] := ldr.allocatePRI(ldr#cursorSize) ' Allocate cursor control block
    if not vgaInit.start(vgaPinGroup)                      ' Now start up the VGA driver
      abort false
    long[ldr#keybdPtr]  := ldr.allocatePRI(ldr#keybdSize)  ' Allocate work area for keyboard
    if not keyInit.start(ps2KeyData,ps2KeyClock)           ' Now start up the keyboard driver
      abort false
    if not ldr.start(ldrInit.getEntryPoint)                ' Now start up the I2C driver
      abort false                                         
    long[ldr#userPtr] := ldr#noSuchAddr
    long[ldr#initMarker] := ldr#uniqueMark                 ' Basic initialization complete
    waitcnt(clkfreq * 2 + cnt)                             ' Wait for keyboard to initialize
    text.out(text#Cr)
  if not key.present
    text.str(string("--> keyboard not present, can't continue"))
    long[ldr#initMarker] := 0
    abort false
  start

PUB start
  waitcnt(clkfreq + cnt)                 ' Wait for simulator to initialize
  start1

PUB start1

  start2

PUB start2
  
  repeat
    text.out(0)
    showMenu
    case selection := getChoice
      "T":
        vgainit.stop
        key.getkey
        vgainit.start(vgapingroup)
      "S":
        repeat z from 0 to 39
          text.out4($13,z,240,10)
        text.out(0)
        text.str(string("This option gives a brief explanation of the most common operators used",13,13))
        text.str(string("in the SPIN language.  In some cases explanatory examples are given.",13,13))
        text.str(string("Memory limitations preclude the inclusion of of all SPIN operators.  If",13,13))
        text.str(string("the operator you search for returns a 'Not found' please refer to the",13,13))
        text.str(string("Propeller Manual",13,13))
        text.str(string("***********************************************************************",13,13))
        text.str(string("Press any key to continue"))
        key.getkey
        text.out(0)
        readop     'Line 105

PUB setxy(n,m)
  text.out(2)
  text.out(n)
  text.out(m)

PRI showMenu
  repeat z from 0 to 39
    text.out4($13,z,240,10)
  text.out(13)
  text.str(string(19,1,10,240,"                     Menu for Data Test V1.00",13,13,13))
  text.str(string(" T - Toggle VGA display",13,13))
  text.str(string(" S - Search for operator meaning",13,13))
  setxy(2,16)
  text.str(string("    Select: "))

PRI getChoice : selectedOption
{ Accept menu choice. Returns uppercase letter. }
  'key.clearkeys
  repeat
    repeat until selectedOption := key.key
    if (selectedOption => "a") and (selectedOption =< "z")                         
      selectedOption -= 32    '32 = ("a" - "A")                     'Convert letters to uppercase
  until (lookdown (selectedOption : "TS"))
  
PUB readop | p
  p := 0
  text.str(string("Key in operator, followed by Enter:    "))
  repeat until  (buffer[p] :=key.getkey) == 13 
    text.out(buffer[p++])
  buffer[p] := 0
  text.out(13)
  text.str(@buffer) 
  testop(@buffer)
  text.out(13)
  

PUB testop(nameAddr) | index
  index := lookupop(nameAddr,@opNo)
  if index == -1
    text.out(13) 
    text.str(string(19,9,62,10,"Operator not found",13))
    text.str(string(19,10,62,10,"Press any key to return to Menu",13))
      if key.getkey
        start2
  text.out(13)
  text.out(0)
  setxy(0,4)
  text.str(@@opDesc[index])
  text.str(string(13,13,"Press C to continue operator search",13))
  text.str(string(13,13,"Press R to return to Menu",13))
  if key.newkey == "c"
    text.out(0)
    readop
  else
    start2
    
PUB lookupop(nameAddr, tblAddr) : index
  index := 0
  repeat until byte[tblAddr] == 0
    if strcomp(nameAddr,tblAddr)
      return
    index++
    tblAddr += strsize(tblAddr) + 1
  return -1              
      


DAT

                
opNo  
        byte  "<<",0
        byte  ">>",0
        byte  "+=",0
        byte  "-=",0
        byte   "--",0
        byte  ":=",0
        byte  "==",0
        byte  "++",0
        byte  "*=",0
        byte  "/=",0
        byte  "//",0
        byte  "^^",0
        byte  "||",0
        byte  "?",0      '13
        byte  "|<",0
        byte  ">|",0   '15
        byte  "<-",0
        byte  "->",0  '17  
        byte  "&",0   '18  2911 longs free
        byte  "|",0   '19  2833 longs free
        byte  "^",0   '20
        byte  "!",0   '21
        byte "and",0  '22
        byte "or",0   '23
        byte "not",0  '24
        byte "<>",0   '25
        byte "=<",0   '26
        byte "=>",0   '27
        byte "@",0    '28
        byte "@@",0   '29
        byte "~~",0   '30
        byte "~",0    '31
        
opDesc     word  @op0Desc,@op1Desc,@op2Desc,@op3Desc,@op4Desc
           word  @op5Desc,@op6Desc,@op7Desc,@op8Desc,@op9Desc
           word  @op10Desc,@op11Desc,@op12Desc,@op13Desc,@op14Desc
           word  @op15Desc,@op16Desc,@op17Desc,@op18Desc,@op19Desc
           word  @op20Desc,@op21Desc,@op22Desc,@op23Desc,@op24Desc
           word  @op25Desc,@op26Desc,@op27Desc,@op28Desc,@op29Desc
           word  @op30Desc,@op31Desc
             

op0Desc    byte "This is the shift left operator, and is followed by a number.",13
           byte "X<<1 means the value of X is shifted one bit to the left.  Excess left",13
           byte "hand bits are dropped off and one 0s are added to the right end.",13,13
           byte 19,4,62,10,"Example:  X = 8 = 0000_1000.  X<<1 = 0001_0000 = 16.",13,0
           
op1desc    byte "This is the shift right operator - the opposite of the shift left.",13,13
           byte 19,2,62,10,"Example:  X = 8 = 0000_1000.  X>>1 = 0000_0100 = 4.",13,0

op2desc    byte 13,"X += Y is the same as saying X = X + Y.",0

op3Desc    byte 13,"X -= Y is the same as saying X = X - Y.",0

op4Desc    byte 13,"' -- ' is the decrement operator, either pre or post.  Let's say X := 10",13,13
           byte $13,3,62,10,"Example of pre-decrement:  Y := --X + 2",13,13
           byte "In this case X is decremented by 1 before the calculation is performed.",13
           byte $13,6,240,10,"So...Y := --X + 2 = Y := 9 + 2 = 11.",13,13
           byte $13,8,62,10,  "Example of post-decrement:  Y := X-- + 2.",13,13
           byte "In this case, Y := 10 + 2 = 12, and then  X is decremented to 9",13
           byte "In both cases X becomes 9 and retains that value until otherwise changed", 13, 13,0 

op5Desc    byte "' := ' is the assignment equality operator.  X := 5 means you are ",13
           byte "unconditionally assigning the value of 5 to the variable X.",13,13,0

op6Desc    byte "' == ' is the equality operator for conditional statements.",13,13
           byte $13,2,62,10,"Example:  If you want to say 'if X is equal to Y, then you would",13
           byte $13,3,62,10,"write:  if X == Y.",13,13,0

op7desc    byte "' ++ ' is the increment operator, either pre or post.  Say X := 10.",13,13
           byte $13,2,62,10,"Example of pre-increment:  Y := ++X + 2",13,13
           byte "This means means increment X by 1 before the calculation is performed.",13
           byte "So...Y := ++X + 2 = Y:= 11 + 2 = 13",13,13
           byte $13,7,62,10,"Example of post-increment:  Y := X++ + 2",13,13
           byte "This means increment X by 1 after the calculation.  ",13
           byte "So Y := X++ + 2 = Y :- 10 + 2 = 12.  In both cases X becomes 11",13
           byte "and retains that value until otherwise changed.",13,13,0
           
op8Desc    byte "X *= 20 is the same thing as saying X := X * 20.",13,13,0

op9Desc    byte "X /= 20 is the same thing as saying X:= X / 20.",13,13,0

op10desc   byte "' // ' is the modulus operator.  In the expression ",13,13
           byte "Y := X // 3, let X = 22.  The expression then translates",13,13
           byte "to Y = remainder after 22 is divided by 3.  22 / 3 = 7 + 1",13,13
           byte "so Y := 1.",13,13,0

op11Desc   byte 13,"' ^^ ' is the square root operator.  X := ^^Y means X equals the square",13,13
           byte "root of Y.  Be aware the result will be an integer unless you are using",13,13
           byte "a floating point object.",13,13,0

op12Desc   byte "' || ' os the Absolute Value operator",13,13
           byte "The Absolute Value operator, also called Absolute, returns the absolute",13,13
           byte "value (the positive form) of a number. Absolute Value can be used in both",13,13
           byte "variable and constant expressions.",13,13
           byte 19,8,62,10,"Example: X := || Y",13,13
           byte "If Y = -20 then X = 20.  The negative sign of Y is disregarded.",13,0
            
op13Desc   byte "' ? '  is the operator to generate a random number.",13,13
           byte 19,2,62,10,"Example:  X := ?Y",13,13
           byte "In this example, Y is the value from which a random 'seed' is generated.",13,13
           byte "X must be a LONG variable.  Here is a short method that will generate 10",13,13
           byte "random numbers:",13,13
           byte 19,10,180,10,"PUB random",13,13
           byte 19,12,180,10,"  text.out(0)",13
           byte 19,13,180,10,"  num := 4",13
           byte 19,14,180,10,"  repeat 10",13
           byte 19,15,180,10,"    text.dec(z)",13
           byte 19,16,180,10,"    text.out(13)",13
           byte 19,17,180,10,"  text.str(string(Press any key to continue))",13
           byte 19,18,180,10,"  key.getkey",13,13
           byte "The 'Press any key' statement must, of course, be in quotes in your",13,13
           byte "method.  If the random numbers are too long, try right shifting - z>>N",13,13
           byte "using different values for N.  For excample, z>>24 will give you a list",13,13
           byte "of numbers with a max of 3 digits.",13,0

op14Desc   byte "' |< ' is the Bitwise Decode operator.  It decodes a value (0 to 31) into a ",13,13
           byte "32-bit long value with a single bit set high corresponding to the bit",13,13
           byte "position of the original value. It can not be used in floating point ",13,13
           byte "expressions.",13,13
           byte 19,8,62,10,"Example:  Pin := |<PinNum",13,13
           byte 19,10,62,10,"    If PinNum is 3, Pin = %00000000 00000000 00000000 00001000.",13,13
           byte 19,12,62,10,"    If PinNum is 31, Pin = %10000000 00000000 00000000 00000000.",13,0      

op15Desc   byte "' >| ' is the Bitwise Encode operator.  It encodes a 32-bit long value into",13,13
           byte "the value (0 to 32) that represents the highest bit set, plus 1.  It can",13,13
           byte "not be used in floating point expressions.",13,13
           byte 19,6,62,10,"Example:  PinNum := >|Pin",13,13
           byte 19,8,62,10,"    If Pin is %00000000 00000000 00000000 10000000, PinNum = 8; bit 7 is set.",13,13
           byte 19,10,62,10,"    If Pin is %00000000 00010011 00010010 00100000, PinNum = 21;",13,13
           byte 19,12,62,10,"      bit 20 is the highest bit set.",13,0

op16Desc   byte "' <- ' is the Bitwise Rotate Left operator.  It is similar to the Bitwise",13,13
           byte "Shift Left operator, except that the MSBs (leftmost bits) are rotated back",13,13
           byte "around to the LSBs (rightmost bits).  It can not be used in floating",13,13
           byte "point expressions.",13,13
           byte 19,8,62,10,"Example:  X := Y <- 1.  If Y = 1111 1100 then X = 1111 1001.",13,0

op17Desc   byte "' -> ' is the Bitwise RotateRight operator.  It is similar to the Bitwise",13,13
           byte "Shift Right operator, except that the LSBs (rightmost bits) are",13,13
           byte "rotated back around to the MSBs (leftmost bits).  It can not be",13,13
           byte "used in floating point expressions.",13,13
           byte 19,8,62,10,"Example:  X := Y -> 5.  If Y = 1111 1001 then X = 1111 1100.",13,0

op18Desc   byte "' & ' is the Bitwise AND operator.  It can not be used in floating",13,13
           byte "point expressions.",13,13
           byte 19,4,62,10,"Example:  X := %0010 1100 & %0000 1111.  When these two",13,13
           byte 19,6,62,10,"    values are ANDed, a 1 is returned ONLY if both bits = 1",13,13
           byte 19,8,62,10,"    otherwise a 0 is returned.  In this example, X = %0000 1100",13,0

op19Desc   byte "' | ' is the Bitwise OR operator.  This operator can not be used",13,13
           byte "in floating point expressions.  Whem two values are ORed together,",13,13
           byte "a 0 will be returned if corresponding bits are both 0, otherwise,",13,13
           byte "a 1 will be returned.",13,13
           byte 19,8,62,10,"Example:  X := %0010 1100 | %0000 1111.  In this case,",13,13
           byte 19,10,62,10,"    X := %0010 1111.",13,0

op20Desc   byte "' ^ ' is the Bitwise XOR operator.  XOR means Exclusive OR.  If",13,13
           byte "bits are identical, a 0 will be returned.  If they are different,",13,13
           byte "a 1 will be returned.",13,13
           byte 19,6,62,10,"Example:  X := X := %0010 1100 | %0000 1111.  When these",13,13
           byte 19,8,62,10,"    two values are XORed, X := %0010 0011.",0

op21Desc   byte "' ! ' is the Bitwise NOT operator.  This operator inverts",13,13
           byte " - or toggles - the bits of a value.",13,13
           byte 19,4,62,10,"Example:  X := !%0010 1100.  X := %1101 0011.",13,13
           byte "The ! operator can also be used to toggle the state of an output",13,13
           byte "pin.",13,13
           byte 19,10,62,10,"Example:  !outa[dirx].  This statement will toggle the",13,13
           byte 19,12,62,10,"    state of dirx - if it was high it will go low.  If it",13,13
           byte 19,14,62,10,"    was low it will go high.",13,0 

op22Desc   byte "' AND ' is the Boolean AND operator.  It compares two values, and if",13,13
           byte "both are non-zero it return a TRUE (-1).  If either value is zero, it",13,13
           byte "returns a FALSE (0).",13,13
           byte 19,6,62,10,"Example:  X := Y AND Z.  If both Y and Z are non-zero,",13,13
           byte 19,8,62,10,"    then X := -1.  If Y or Z = 0, then X:= 0.",13,13,0
          
op23Desc   byte "' OR ' is the Boolean OR operator.  It compares two values and if",13,13
           byte "either is non-zero, it returns a TRUE (-1).  If both values = 0,",13,13
           byte "then it returns a FALSE (0).",13,13
           byte 19,6,62,10,"Example:  X := Y OR Z.  If either Y or Z is non-zero,",13,13
           byte 19,8,62,10,"    then X := -1.  If both = 0 then X := 0.",13,0

op24Desc   byte "' NOT '  is the Boolean NOT operator.  It returns the Boolean",13,13
           byte "opposite of Y.",13,13
           byte 19,4,62,10,"Example:  X := NOT Y.  If Y = 0, then X := -1.",13,13
           byte 19,6,62,10,"    If Y is non=zero, then X := 0.",13,0

op25Desc   byte "' <> ' is the Not Equal operator.  It returns a TRUE (-1) if the",13,13
           byte "values are not equal, or returns a FALSE if the valyes are equal.",13,13
           byte "<> can be used in two different ways.",13,13
           byte 19,6,62,10,"Example 1:  X := Y<>Z.  If Y and Z are different values,",13,13
           byte 19,8,62,10,"    then X := -1.  If Y and Z are the same, then X := 0.",13,13
           byte 19,10,62,10,"Example 2:  if X <> 25 (conditional statement)",13,13
           byte 19,12,62,10,"               task1",13,13
           byte 19,14,62,10,"            else start",13,0

op26Desc   byte "' =< ' means 'equal to or less than'",13,13
           byte 19,2,62,10,"Example:  if X =< 100.  Translation - if X is equal to",13,13
           byte 19,4,62,10,"    or less than 100",13,0

op27Desc   byte "' => ' means 'equal to or greater than'",13,13
           byte 19,2,62,10,"Example:  if X => 100.  Translation - if x is equal to",13,13
           byte 19,4,62,10,"    or greater than 100",13,0

op28Desc   byte "' @ '  is the Symbol Address operator.  Suppose you had a list",13,13
           byte " in your DAT block like this:",13,13
           byte 19,4,62,10,"DAT",13,13
           byte 19,6,62,10,"op26Desc   byte  =< ' means equal to or less than",13,13   
           byte 19,8,62,10,"           byte Example:  if X => 100.  Translation - if x",13,13 
           byte 19,10,62,10,"           byte is equal to or less than 100",13,13
           byte "If you wanted to display all three lines on your monitor,",13,13
           byte " you would write ' text.str(@op26Desc) '.  Please note that",13,13
           byte "the above example is not in precisely the proper format.  It",13,13
           byte "had to be modified slightly to permit display here.  DAT lists",13,13
           byte "must end with a 0.",13,0

op29Desc   byte "' @@ ' is the Object Address Plus Symbol operator.  Please see",13,13
           byte "Page 279 of the Propeller Manual for an explanation and examples.",13,0

op30Desc   byte "~~ is the post-Set operator.  Writing dira[pin]~~ is the same as saying: ",13,13
           byte 19,6,62,10,"dira[pin] := 1",13,0

op31Desc   byte "~ is the Post-Clear operator.  Writing dira[pin]~ is the same as saying:",13,13
           byte 19,6,62,10,"dira[pin] := 0",13,0






                
'****************************************************************************